#! /usr/bin/perl

#
# Test dungeon for SE-3, VWA, SoSe 2004
# 02-MAY-2004, B. Ulmann, fecit
# 06-MAY-2004, B. Ulmann,  Display objects in rooms, objects belonging to a
#                         particular user, users in a room.
#                          Changed startup processing due to new login script.
#

# Be sure to warn on undeclared variables, etc.
use strict;
use warnings;
use DBI;
use POSIX;

# $debug = 1 results in some debug output - this is a global variable! :-)
my $debug = 0;

# Connect to database
sub conn
{
    my ($driver, $database, $hostname, $port, $user, $password) = @_;

    if ($debug)
    {
	print "DEBUG: conn ()<br>\n";
    }

    my $dsn = "DBI:$driver:database=$database;host=$hostname;port=$port";
    my $dbh = DBI -> connect ($dsn, $user, $password); # Connect to database

    if (!$dbh)
    {
	print "<b>conn (): Could not connect to database!";
	exit;
    }

    if ($dbh -> err)
    {
	print "<b>conn (): Connection error: ", $dbh -> errstr;
	exit;
    }

    print "<hr>\n" if $debug;

    return $dbh;
}

# Print all information about a location specified by a location number $lnr
sub display_location
{
    my ($dbh, $lnr, $picture_prefix, $unr) = @_;

    if ($debug)
    {
	print "DEBUG: display_location (): lnr = $lnr, picture_prefix: $picture_prefix, unr=$unr<br>\n";
    }

    my $statement = "select name, description, picture from locations where lnr = $lnr";
    my $sth = $dbh -> prepare ($statement);
    $sth -> execute;

    if ($sth -> err)
    {
	print "<b>display_location (): Error: ", $sth -> errstr;
	exit;
    }

    my ($name, $description, $picture);
    $sth -> bind_columns (\$name, \$description, \$picture);

    if (!$sth -> fetch)
    {
	print "<b>display_location (): Nothing to fetch!";
	exit;
    }

    $sth -> finish;

    $picture = $picture_prefix . $picture if $picture;

    my $user_name = get_user_name ($dbh, $unr);
    my $tmstmp = strftime ("%Y-%m-%d %H:%M:%S", localtime);
    print "<center><table><tr><td>Login: $user_name</td><td>Current time: $tmstmp</td></tr></table></center><hr>
<center><h1>$name</h1></center><hr>\n";

    print "<center><table valign=\"top\"><tr>\n";
    get_users ($dbh, $lnr, $unr);
    print "<td><img src=\"$picture\"></center></td>\n" if $picture;
    get_objects ($dbh, $lnr, 0);
    print "</tr></table></center>";

    print "<center>$description<hr></center>\n";
    print "<center>\n";
    get_objects ($dbh, $unr, 1);
    print "</center>\n";
}

# Create a button for each possible direction to go to
sub display_directions
{
    my ($dbh, $lnr, $cgi, $unr) = @_;

    if ($debug)
    {
	print "DEBUG: display_directions ():lnr = $lnr<br>\n";
    }	

    my $statement = "select d.name
from connections c, directions d
where c.from_lnr = $lnr
and c.from_dnr = d.dnr";
    my $sth = $dbh -> prepare ($statement);
    $sth -> execute;

    if ($sth -> err)
    {
	print "<b>display_directions (): Error: ", $sth -> errstr;
	exit;
    }

    my $direction;
    $sth -> bind_columns (\$direction);

    print "<form action=\"$cgi\" method=\"POST\"><center><table><tr>\n";
    while ($sth -> fetch ())
    {
	print "<td><input name=\"direction\" type=\"submit\" value=\"$direction\"></td>\n";
    }
    $sth -> finish;

    print "<td><input name=\"lnr\" type=\"hidden\" value=\"$lnr\"></td>
           <td><input name=\"unr\" type=\"hidden\" value=\"$unr\"></td>\n";
    print "</tr></table></center></form>\n";

    print "<hr>\n" if $debug;
}

#  Get next location number. This is determined from the current location number
# and the name of the direction to go to. 
sub get_lnr
{
    my ($dbh, $lnr, $direction) = @_;

    if ($debug)
    {
	print "DEBUG: get_lnr (): lnr = $lnr, direction = $direction<br>\n";
    }

    my $statement = "select c.to_lnr
from connections c, directions d
where c.from_dnr = d.dnr
and d.name = \"$direction\"
and c.from_lnr = $lnr";
    my $sth = $dbh -> prepare ($statement);
    $sth -> execute;

    if ($sth -> err)
    {
	print "<b>get_lnr (): Error: ", $sth -> errstr;
	exit;
    }

    my $next_location;
    $sth -> bind_columns (\$next_location);

    if (!$sth -> fetch)
    {
	print "<b>display_location (): Nothing to fetch!";
	exit;
    }

    $sth -> finish;

    return $next_location;
}

# Create a list of all users who are currently in the same room as the active player.
sub get_users
{
    my ($dbh, $lnr, $unr) = @_;

    if ($debug)
    {
	print "DEBUG: get_users (): lnr = $lnr, unr = $unr\n";
    }

    my $statement = "select nickname from users where last_lnr=\"$lnr\" and unr!=$unr";
    my $sth = $dbh -> prepare ($statement);
    $sth -> execute;

    if ($sth -> err)
    {
	print "<b>get_users (): Error: ", $sth -> errstr;
	exit;
    }

    my $name;
    $sth -> bind_columns (\$name);

    print "<td><table>\n";
    while ($sth -> fetch ())
    {
	print "
 <tr>
  <td>
   $name
  </td>
 </tr>
";
    }
    print "</table></td>\n";

    $sth -> finish;
}

# Create a list of all objects at a location or carried by a user.
#  If $location_type is equal 0 then a single column table is generated to 
# be displayed on the right of the location picture. Otherwise (i.e.
# the objects belong to a specific person) a horizontal table with 
# only one row is generated to be displayed at the bottom of the 
# location picture.
sub get_objects
{
    my ($dbh, $actual_location, $location_type) = @_;

    if ($debug)
    {
	print "DEBUG: get_objects (): actual_location = $actual_location,location_type: $location_type<br>\n";
    }

    my $statement = "select name, picture_thumb, picture 
from objects 
where actual_location=\"$actual_location\"
and actual_location_type=\"$location_type\"";
    my $sth = $dbh -> prepare ($statement);
    $sth -> execute;

    if ($sth -> err)
    {
	print "<b>get_objects (): Error: ", $sth -> errstr;
	exit;
    }

    my ($name, $picture_thumb, $picture);
    $sth -> bind_columns (\$name, \$picture_thumb, \$picture);

    if ($location_type == 0) # vertical table
    {
	print "<td><table border=\"1\" bordercolor=\"#FFFFFF\">\n";
	while ($sth -> fetch ())
	{
	    print "
   <tr>
    <td>
     $name
    </td>
   </tr>
";
	}
	print "</table></td>\n";
    }
    else # horizontal table
    {
	print "<table border\"1\" bordercolor=\"#FFFFFF\"><tr>\n";
	while ($sth -> fetch ())
	{
	    print "
    <td>
     $name
    </td>
";
	}
	print "</tr></table><hr>\n";
    }

    $sth -> finish;
}

#  This routine is called as the first task after determination of the current
# location number $lnr at the begin of this script. It stores this $lnr in 
# the appropriate row of the table "users" in the database as well as the
# timestamp of the current action.
sub update_user_data 
{
    my ($dbh, $unr, $lnr) = @_;

    if ($debug)
    {
	print "Debug: update_user_data (): unr=$unr, lnr=$lnr<br>\n";
    }

    my $tmstmp = strftime ("%Y%m%d%H%M%S", localtime);
    my $statement = "update users
set last_lnr = $lnr, status=1, last_action=$tmstmp
where unr = $unr";

    my $sth = $dbh -> prepare ($statement);
    $sth -> execute;

    if ($sth -> err)
    {
	print "<b>Could not update last_lnr. Error: ", $sth -> errstr;
	$sth -> finish;
	exit;
    }

    $sth -> finish;
}

# Get the name of a user identified by his $unr.
sub get_user_name
{
    my ($dbh, $unr) = @_;

    if ($debug)
    {
	print "Debug: get_user_name (): unr=$unr<br>\n";
    }

    my $statement = "select nickname from users where unr=$unr";
    my $sth = $dbh -> prepare ($statement);
    $sth -> execute;

    if ($sth -> err)
    {
	print "<b>Could not determine user name. Error: ", $sth -> errstr;
	exit;
    }

    my $name;
    $sth -> bind_columns (\$name);

    if (!$sth -> fetch)
    {
	print "<b>get_user_name (): Nothing to fetch!";
	$sth -> finish;
	exit;
    }

    $sth -> finish;

    return $name;
}

#########################################################################################
#
# main program
#
#########################################################################################

# Set access parameters:
my $driver = "mysql";
my $database = "dungeon";
my $hostname = "localhost";
my $port = 3306;
my $user = "dungeon";
my $password = "keeper";
my $picture_prefix = '';
my $cgi = "d1.pl"; # Name of the cgi to be called

# Some more variables
my $lnr = 19;      # Initial location number

# first_run = 1 denotes that the script was called for the first time
my $first_run = 1;

# Prepare the output of HTML code:
print "Content-type: text/html

<html>
 <body>
";

if ($debug) # We are running in debug mode!
{
    print "<center><b>Running in debug mode!</b></center><hr>\n";
}

# Read parameters from GET or POST access:
my ($buffer, @pairs, $pair, $name, $value, %FORM);

# Read in text
$ENV{'REQUEST_METHOD'} =~ tr/a-z/A-Z/;
if ($ENV{'REQUEST_METHOD'} eq "POST")
{
    read (STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
}
else # Not POST -> GET :-)
{
    $buffer = $ENV{'QUERY_STRING'};
}

# Split information into name/value pairs
@pairs = split(/&/, $buffer);
foreach $pair (@pairs)
{
    ($name, $value) = split(/=/, $pair);
    $value =~ tr/+/ /;
    $value =~ s/%(..)/pack("C", hex($1))/eg;

    $FORM{$name} = $value;
}

# Dump parameters:
if ($debug)
{
    print "DEBUG: Parameters: first_run = $first_run<br><ul>\n";
    foreach my $key (keys %FORM)
    {
	print "<li>$key = $FORM{$key}\n";
    }
    print "</ul>\n";
}

if ($FORM{'lnr'})
{
    $first_run = 0;
}

my $unr = $FORM{'unr'};
if (!$unr)
{
    print "<center><h1><font color=\"red\">Illegal call of $cgi!</font></h1></center>";
    exit;
}


# Connect to database
my $dbh = conn ($driver, $database, $hostname, $port, $user, $password);

if (!$first_run and $FORM{'login'} != 1) # Not the first run - so determine the actual room number
{
    $lnr = get_lnr ($dbh, $FORM{'lnr'}, $FORM{'direction'});
}

update_user_data ($dbh, $unr, $lnr);
display_location ($dbh, $lnr, $picture_prefix, $unr);
display_directions ($dbh, $lnr, $cgi, $unr);

print " </body>
</html>
";

$dbh -> disconnect;








